#include "box.h"

#include <cassert>  // assert
#include <string>   // std::string

using namespace ljts;

std::istream& operator>>(std::istream& is, Box& b)
{
   b.read_positions(&is);
   return is;
}

void Box::read_positions(std::istream* is)
{
   int N;
   *is >> N;
   assert(N > 0);
   
   std::string skip;
   std::getline(*is, skip);
   std::getline(*is, skip);
   std::cout << "# Comment line: " << skip << "\n";
   
   for(int i = 0; i < N; i++)
   {
      Molecule mi(is);
      mi.set_new_id();
      this->molecules[mi.get_id()] = mi;
   }
}

void Box::read_velocities(std::istream* is)
{
   int N;
   *is >> N;
   assert(N == this->molecules.size());
   
   std::string skip;
   std::getline(*is, skip);
   std::getline(*is, skip);
   std::cout << "# Comment line: " << skip << "\n";

   for(auto i = this->molecules.begin(); i != this->molecules.end(); i = std::next(i))
      i->second.read_velocity(is);
}

double Box::E_kin()
{
   double sum = 0.0;
   for(auto i = this->molecules.begin(); i != this->molecules.end(); i = std::next(i))
      sum += i->second.E_kin();
   return sum;
}

double Box::E_pot()
{
   double sum = 0.0;
   for(auto i = this->molecules.begin(); i != this->molecules.end(); i = std::next(i))
      for(auto j = std::next(i); j != this->molecules.end(); j = std::next(j))
      {
         // std::cout << "# " << i->second.get_id() << " " << j->second.get_id() << "\t" << i->second.interaction(&(j->second)) << "\n";
         sum += i->second.interaction(&(j->second));
      }
   return sum;
}
