#include "molecule.h"

#include <cmath>    // functionality around NaN

using namespace ljts;

namespace
{
   int num_molecules = 0;   
   float r2cut = 6.25;  // 2.5 squared
   float shift = 0.01631689;  // potential difference by which LJTS is shifted
}

std::istream& operator>>(std::istream& is, Molecule& i)
{
   i.read_position(&is);
   return is;
}

void Molecule::set_new_id()
{
   this->id = num_molecules++;
}

void Molecule::read_position(std::istream* is)
{
   char inchar;
   
   do is->get(inchar);
   while((inchar == ' ') || (inchar == '\t') || (inchar == '\n'));
   // now ignore the first item, which is the XYZ file atom code

   double qx = NAN;
   *is >> qx;
   assert(!std::isnan(qx));

   double qy = NAN;
   *is >> qy;
   assert(!std::isnan(qy));
   
   double qz = NAN;
   *is >> qz;
   assert(!std::isnan(qz));

   this->set_position(qx, qy, qz);
}

void Molecule::read_velocity(std::istream* is)
{
   char inchar;
   
   do is->get(inchar);
   while((inchar == ' ') || (inchar == '\t') || (inchar == '\n'));
   // now ignore the first item, which is the XYZ file atom code

   double vx = NAN;
   *is >> vx;
   assert(!std::isnan(vx));

   double vy = NAN;
   *is >> vy;
   assert(!std::isnan(vy));
   
   double vz = NAN;
   *is >> vz;
   assert(!std::isnan(vz));

   this->set_velocity(vx, vy, vz);
}

double Molecule::interaction(Molecule* j)
{
   double r[3] = {
      j->get_position(0) - this->q[0],
      j->get_position(1) - this->q[1],
      j->get_position(2) - this->q[2]
   };
   
   // reduced square radius
   float rr = r[0]*r[0] + r[1]*r[1] + r[2]*r[2];
   
   // cutoff at r_cut = 2.5
   if(rr >= r2cut) return 0;

   // potential energy from the LJTS pair potential
   double invrr = 1.0/rr;
   double inv_r06 = invrr*invrr*invrr;
   double inv_r12 = inv_r06 * inv_r06;

   return 4.0 * (inv_r12 - inv_r06) + shift;
}
