#pragma once

#include <cassert>   // assert
#include <iostream>  // std::istream

namespace ljts
{
   class Molecule;
}
std::istream& operator>>(std::istream& is, ljts::Molecule& i);
   
namespace ljts
{
   class Molecule
   {
   public:
      Molecule() = default;
      Molecule(std::istream* is) {
         *is >> *this;
      }
      
      int get_id() { return this->id; }
      double get_position(int d) {
         assert(d == 0 || d == 1 || d == 2);
         return this->q[d];
      }
      
      void set_new_id();
      void set_position(double qx, double qy, double qz) {
         this->q[0] = qx;
         this->q[1] = qy;
         this->q[2] = qz;
      }
      void set_velocity(double vx, double vy, double vz) {
         this->v[0] = vx;
         this->v[1] = vy;
         this->v[2] = vz;
      }
      void read_position(std::istream* is);
      void read_velocity(std::istream* is);
      
      inline double E_kin() { return 0.5*(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]); }
      double interaction(Molecule* j);

   private:
      int id{};
      double q[3]{};
      double v[3]{};
   };
}

