#include <box.h>

#include <cassert>   // assert
#include <fstream>   // std::ifstream
#include <iostream>

namespace
{   
   std::string default_infile = "config.dat";
}

int main(int argc, char** argv)
{
   assert(argc > 1);

   /*
    * read configuration
    */   
   std::ifstream fconf(argv[1]);
   if(!fconf)
   {
      std::cout << "File '" << argv[1] << "' could not be opened.\n\n";
      return EXIT_FAILURE;
   }
   std::cout << "# Configuration input file:  " << argv[1] << " (opened).\n";
   
   ljts::Box b;
   fconf >> b;
   fconf.close();
   std::cout << "# Configuration file processed.\n";
   
   if(argc > 2)
   {
      /*
       * read velocities
       */
      std::ifstream fmom(argv[2]);
      if(!fmom)
      {
         std::cout << "File '" << argv[2] << "' could not be opened.\n\n";
         return EXIT_FAILURE;
      }
      std::cout << "# Momentum input file:  " << argv[2] << " (opened).\n";
   
      b.read_velocities(&fmom);
      fmom.close();
      std::cout << "# Momentum file processed.\n";
   }
   
   /*
    * compute and output energy
    */
   double E_kin = b.E_kin();
   std::cout << "E_kin = " << E_kin << ".\n"; 
   double E_pot = b.E_pot();
   std::cout << "E_pot = " << E_pot << ".\n"; 
   std::cout << "E_kin + E_pot = " << E_kin + E_pot << ".\n";
}
